from collections.abc import Sequence
from typing import overload

from lxml.etree import _Element

def add_ids(node: _Element, ids: Sequence[str]) -> None: ...
@overload
def find_child(parent: _Element, name: str) -> _Element | None: ...
@overload
def find_child(parent: _Element, name: str, namespace: str = ...) -> _Element | None: ...
@overload
def find_node(node: _Element, name: str) -> _Element | None: ...
@overload
def find_node(node: _Element, name: str, namespace: str = ...) -> _Element | None: ...
@overload
def find_parent(node: _Element, name: str) -> _Element | None: ...
@overload
def find_parent(node: _Element, name: str, namespace: str = ...) -> _Element | None: ...
